/*
 * Author: Kipp Cannon <kipp@gravity.phys.uwm.edu>
 * Revision: $Id: Sequence.m4,v 1.3 2005/06/20 17:43:40 kipp Exp $
 */

#include <string.h>
#include <lal/Date.h>
#include <lal/LALDatatypes.h>
#include <lal/LALStdlib.h>
#include <lal/LALStatusMacros.h>
#include <lal/LALErrno.h>
#include <lal/LALError.h>
#include <lal/Sequence.h>
#include <lal/XLALError.h>

NRCSID(SEQUENCEC, "$Id: Sequence.m4,v 1.3 2005/06/20 17:43:40 kipp Exp $");

/*
 * Shift the bytes in the buffer buff, whose length is length, count bytes to
 * higher addresses.  If the magnitude of count is greater than or equal to
 * that of length, then nothing is done.
 */

static void memshift(void *buff, size_t length, int count)
{
	if(count >= 0) {
		if(length > (size_t) count)
			memmove((char *) buff + count, buff, length - count);
	} else {
		if(length > (size_t) -count)
			memmove(buff, (char *) buff - count, length + count);
	}
}


/*
 * Begin library functions...
 */




void XLALDestroyREAL4Sequence (
	REAL4Sequence *sequence
)
{
	if(sequence)
		LALFree(sequence->data);
	LALFree(sequence);
}


REAL4Sequence *XLALCreateREAL4Sequence (
	size_t length
)
{
	static const char *func = "XLALCreateREAL4Sequence";
	REAL4Sequence *new;
	REAL4 *data;

	new = LALMalloc(sizeof(*new));
	data = LALMalloc(length * sizeof(*data));
	if(!new || !data) {
		LALFree(new);
		LALFree(data);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	new->data = data;
	new->length = length;

	return(new);
}


REAL4Sequence *XLALCutREAL4Sequence (
	REAL4Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutREAL4Sequence";
	REAL4Sequence *new = NULL;

	if(sequence && sequence->data) {
		new = XLALCreateREAL4Sequence (length);
		if(!new)
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
		memcpy(new->data, sequence->data + first, length * sizeof(*new->data));
	}

	return(new);
}


REAL4Sequence *XLALCopyREAL4Sequence (
	REAL4Sequence *sequence
)
{
	return(XLALCutREAL4Sequence (sequence, 0, sequence->length));
}


void XLALShiftREAL4Sequence (
	REAL4Sequence *sequence,
	int count
)
{
	if(!sequence || !sequence->data || !count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
/* FIXME: this function does not conform to the XLAL error reporting
 * convention. */
size_t XLALResizeREAL4Sequence (
	REAL4Sequence *sequence,
	int first,
	size_t length
)
{
	if(!sequence || !sequence->data)
		return(0);

	if(length > sequence->length) {
		/* need to increase memory */
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(sequence->data) {
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftREAL4Sequence (sequence, -first);
		} else
			sequence->length = 0;
	} else {
		/* do not need to increase memory */
		XLALShiftREAL4Sequence (sequence, -first);
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(sequence->data)
			sequence->length = length;
		else
			sequence->length = 0;
	}

	return(sequence->length);
}


size_t XLALShrinkREAL4Sequence (
	REAL4Sequence *sequence,
	size_t first,
	size_t length
)
{
	return(XLALResizeREAL4Sequence (sequence, first, length));
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
REAL4 XLALREAL4Sum (
	const REAL4 *data,
	size_t first,
	size_t count
)
{
	REAL4 sum = 0;

	for(data += first; count-- > 0; data++) {
		sum += *data;
	}

	return(sum);
}
#endif


REAL4 XLALREAL4SumSquares (
	const REAL4 *data,
	size_t first,
	size_t count
)
{
	REAL4 sum = 0;

	for(data += first; count-- > 0; data++) {
		sum += *data * *data;
	}

	return(sum);
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
REAL4 XLALREAL4SequenceSum (
	const REAL4Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALREAL4Sum (sequence->data, first, count));
}
#endif


REAL4 XLALREAL4SequenceSumSquares (
	const REAL4Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALREAL4SumSquares (sequence->data, first, count));
}





void XLALDestroyREAL8Sequence (
	REAL8Sequence *sequence
)
{
	if(sequence)
		LALFree(sequence->data);
	LALFree(sequence);
}


REAL8Sequence *XLALCreateREAL8Sequence (
	size_t length
)
{
	static const char *func = "XLALCreateREAL8Sequence";
	REAL8Sequence *new;
	REAL8 *data;

	new = LALMalloc(sizeof(*new));
	data = LALMalloc(length * sizeof(*data));
	if(!new || !data) {
		LALFree(new);
		LALFree(data);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	new->data = data;
	new->length = length;

	return(new);
}


REAL8Sequence *XLALCutREAL8Sequence (
	REAL8Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutREAL8Sequence";
	REAL8Sequence *new = NULL;

	if(sequence && sequence->data) {
		new = XLALCreateREAL8Sequence (length);
		if(!new)
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
		memcpy(new->data, sequence->data + first, length * sizeof(*new->data));
	}

	return(new);
}


REAL8Sequence *XLALCopyREAL8Sequence (
	REAL8Sequence *sequence
)
{
	return(XLALCutREAL8Sequence (sequence, 0, sequence->length));
}


void XLALShiftREAL8Sequence (
	REAL8Sequence *sequence,
	int count
)
{
	if(!sequence || !sequence->data || !count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
/* FIXME: this function does not conform to the XLAL error reporting
 * convention. */
size_t XLALResizeREAL8Sequence (
	REAL8Sequence *sequence,
	int first,
	size_t length
)
{
	if(!sequence || !sequence->data)
		return(0);

	if(length > sequence->length) {
		/* need to increase memory */
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(sequence->data) {
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftREAL8Sequence (sequence, -first);
		} else
			sequence->length = 0;
	} else {
		/* do not need to increase memory */
		XLALShiftREAL8Sequence (sequence, -first);
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(sequence->data)
			sequence->length = length;
		else
			sequence->length = 0;
	}

	return(sequence->length);
}


size_t XLALShrinkREAL8Sequence (
	REAL8Sequence *sequence,
	size_t first,
	size_t length
)
{
	return(XLALResizeREAL8Sequence (sequence, first, length));
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
REAL8 XLALREAL8Sum (
	const REAL8 *data,
	size_t first,
	size_t count
)
{
	REAL8 sum = 0;

	for(data += first; count-- > 0; data++) {
		sum += *data;
	}

	return(sum);
}
#endif


REAL8 XLALREAL8SumSquares (
	const REAL8 *data,
	size_t first,
	size_t count
)
{
	REAL8 sum = 0;

	for(data += first; count-- > 0; data++) {
		sum += *data * *data;
	}

	return(sum);
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
REAL8 XLALREAL8SequenceSum (
	const REAL8Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALREAL8Sum (sequence->data, first, count));
}
#endif


REAL8 XLALREAL8SequenceSumSquares (
	const REAL8Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALREAL8SumSquares (sequence->data, first, count));
}





void XLALDestroyCOMPLEX8Sequence (
	COMPLEX8Sequence *sequence
)
{
	if(sequence)
		LALFree(sequence->data);
	LALFree(sequence);
}


COMPLEX8Sequence *XLALCreateCOMPLEX8Sequence (
	size_t length
)
{
	static const char *func = "XLALCreateCOMPLEX8Sequence";
	COMPLEX8Sequence *new;
	COMPLEX8 *data;

	new = LALMalloc(sizeof(*new));
	data = LALMalloc(length * sizeof(*data));
	if(!new || !data) {
		LALFree(new);
		LALFree(data);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	new->data = data;
	new->length = length;

	return(new);
}


COMPLEX8Sequence *XLALCutCOMPLEX8Sequence (
	COMPLEX8Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutCOMPLEX8Sequence";
	COMPLEX8Sequence *new = NULL;

	if(sequence && sequence->data) {
		new = XLALCreateCOMPLEX8Sequence (length);
		if(!new)
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
		memcpy(new->data, sequence->data + first, length * sizeof(*new->data));
	}

	return(new);
}


COMPLEX8Sequence *XLALCopyCOMPLEX8Sequence (
	COMPLEX8Sequence *sequence
)
{
	return(XLALCutCOMPLEX8Sequence (sequence, 0, sequence->length));
}


void XLALShiftCOMPLEX8Sequence (
	COMPLEX8Sequence *sequence,
	int count
)
{
	if(!sequence || !sequence->data || !count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
/* FIXME: this function does not conform to the XLAL error reporting
 * convention. */
size_t XLALResizeCOMPLEX8Sequence (
	COMPLEX8Sequence *sequence,
	int first,
	size_t length
)
{
	if(!sequence || !sequence->data)
		return(0);

	if(length > sequence->length) {
		/* need to increase memory */
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(sequence->data) {
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftCOMPLEX8Sequence (sequence, -first);
		} else
			sequence->length = 0;
	} else {
		/* do not need to increase memory */
		XLALShiftCOMPLEX8Sequence (sequence, -first);
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(sequence->data)
			sequence->length = length;
		else
			sequence->length = 0;
	}

	return(sequence->length);
}


size_t XLALShrinkCOMPLEX8Sequence (
	COMPLEX8Sequence *sequence,
	size_t first,
	size_t length
)
{
	return(XLALResizeCOMPLEX8Sequence (sequence, first, length));
}


#if 0
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
COMPLEX8 XLALCOMPLEX8Sum (
	const COMPLEX8 *data,
	size_t first,
	size_t count
)
{
	COMPLEX8 sum = {0.0, 0.0};
	

	for(data += first; count-- > 0; data++) {
		sum.re += (*data).re;
		sum.im += (*data).im;
		
	}

	return(sum);
}
#endif


REAL4 XLALCOMPLEX8SumSquares (
	const COMPLEX8 *data,
	size_t first,
	size_t count
)
{
	REAL4 sum = 0;

	for(data += first; count-- > 0; data++) {
		sum += (*data).re * (*data).re + (*data).im * (*data).im;
		
	}

	return(sum);
}


#if 0
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
COMPLEX8 XLALCOMPLEX8SequenceSum (
	const COMPLEX8Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALCOMPLEX8Sum (sequence->data, first, count));
}
#endif


REAL4 XLALCOMPLEX8SequenceSumSquares (
	const COMPLEX8Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALCOMPLEX8SumSquares (sequence->data, first, count));
}





void XLALDestroyCOMPLEX16Sequence (
	COMPLEX16Sequence *sequence
)
{
	if(sequence)
		LALFree(sequence->data);
	LALFree(sequence);
}


COMPLEX16Sequence *XLALCreateCOMPLEX16Sequence (
	size_t length
)
{
	static const char *func = "XLALCreateCOMPLEX16Sequence";
	COMPLEX16Sequence *new;
	COMPLEX16 *data;

	new = LALMalloc(sizeof(*new));
	data = LALMalloc(length * sizeof(*data));
	if(!new || !data) {
		LALFree(new);
		LALFree(data);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	new->data = data;
	new->length = length;

	return(new);
}


COMPLEX16Sequence *XLALCutCOMPLEX16Sequence (
	COMPLEX16Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutCOMPLEX16Sequence";
	COMPLEX16Sequence *new = NULL;

	if(sequence && sequence->data) {
		new = XLALCreateCOMPLEX16Sequence (length);
		if(!new)
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
		memcpy(new->data, sequence->data + first, length * sizeof(*new->data));
	}

	return(new);
}


COMPLEX16Sequence *XLALCopyCOMPLEX16Sequence (
	COMPLEX16Sequence *sequence
)
{
	return(XLALCutCOMPLEX16Sequence (sequence, 0, sequence->length));
}


void XLALShiftCOMPLEX16Sequence (
	COMPLEX16Sequence *sequence,
	int count
)
{
	if(!sequence || !sequence->data || !count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
/* FIXME: this function does not conform to the XLAL error reporting
 * convention. */
size_t XLALResizeCOMPLEX16Sequence (
	COMPLEX16Sequence *sequence,
	int first,
	size_t length
)
{
	if(!sequence || !sequence->data)
		return(0);

	if(length > sequence->length) {
		/* need to increase memory */
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(sequence->data) {
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftCOMPLEX16Sequence (sequence, -first);
		} else
			sequence->length = 0;
	} else {
		/* do not need to increase memory */
		XLALShiftCOMPLEX16Sequence (sequence, -first);
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(sequence->data)
			sequence->length = length;
		else
			sequence->length = 0;
	}

	return(sequence->length);
}


size_t XLALShrinkCOMPLEX16Sequence (
	COMPLEX16Sequence *sequence,
	size_t first,
	size_t length
)
{
	return(XLALResizeCOMPLEX16Sequence (sequence, first, length));
}


#if 0
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
COMPLEX16 XLALCOMPLEX16Sum (
	const COMPLEX16 *data,
	size_t first,
	size_t count
)
{
	COMPLEX16 sum = {0.0, 0.0};
	

	for(data += first; count-- > 0; data++) {
		sum.re += (*data).re;
		sum.im += (*data).im;
		
	}

	return(sum);
}
#endif


REAL8 XLALCOMPLEX16SumSquares (
	const COMPLEX16 *data,
	size_t first,
	size_t count
)
{
	REAL8 sum = 0;

	for(data += first; count-- > 0; data++) {
		sum += (*data).re * (*data).re + (*data).im * (*data).im;
		
	}

	return(sum);
}


#if 0
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
COMPLEX16 XLALCOMPLEX16SequenceSum (
	const COMPLEX16Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALCOMPLEX16Sum (sequence->data, first, count));
}
#endif


REAL8 XLALCOMPLEX16SequenceSumSquares (
	const COMPLEX16Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALCOMPLEX16SumSquares (sequence->data, first, count));
}





void XLALDestroyINT2Sequence (
	INT2Sequence *sequence
)
{
	if(sequence)
		LALFree(sequence->data);
	LALFree(sequence);
}


INT2Sequence *XLALCreateINT2Sequence (
	size_t length
)
{
	static const char *func = "XLALCreateINT2Sequence";
	INT2Sequence *new;
	INT2 *data;

	new = LALMalloc(sizeof(*new));
	data = LALMalloc(length * sizeof(*data));
	if(!new || !data) {
		LALFree(new);
		LALFree(data);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	new->data = data;
	new->length = length;

	return(new);
}


INT2Sequence *XLALCutINT2Sequence (
	INT2Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutINT2Sequence";
	INT2Sequence *new = NULL;

	if(sequence && sequence->data) {
		new = XLALCreateINT2Sequence (length);
		if(!new)
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
		memcpy(new->data, sequence->data + first, length * sizeof(*new->data));
	}

	return(new);
}


INT2Sequence *XLALCopyINT2Sequence (
	INT2Sequence *sequence
)
{
	return(XLALCutINT2Sequence (sequence, 0, sequence->length));
}


void XLALShiftINT2Sequence (
	INT2Sequence *sequence,
	int count
)
{
	if(!sequence || !sequence->data || !count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
/* FIXME: this function does not conform to the XLAL error reporting
 * convention. */
size_t XLALResizeINT2Sequence (
	INT2Sequence *sequence,
	int first,
	size_t length
)
{
	if(!sequence || !sequence->data)
		return(0);

	if(length > sequence->length) {
		/* need to increase memory */
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(sequence->data) {
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftINT2Sequence (sequence, -first);
		} else
			sequence->length = 0;
	} else {
		/* do not need to increase memory */
		XLALShiftINT2Sequence (sequence, -first);
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(sequence->data)
			sequence->length = length;
		else
			sequence->length = 0;
	}

	return(sequence->length);
}


size_t XLALShrinkINT2Sequence (
	INT2Sequence *sequence,
	size_t first,
	size_t length
)
{
	return(XLALResizeINT2Sequence (sequence, first, length));
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
INT2 XLALINT2Sum (
	const INT2 *data,
	size_t first,
	size_t count
)
{
	INT2 sum = 0;

	for(data += first; count-- > 0; data++) {
		sum += *data;
	}

	return(sum);
}
#endif


UINT2 XLALINT2SumSquares (
	const INT2 *data,
	size_t first,
	size_t count
)
{
	UINT2 sum = 0;

	for(data += first; count-- > 0; data++) {
		sum += *data * *data;
	}

	return(sum);
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
INT2 XLALINT2SequenceSum (
	const INT2Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALINT2Sum (sequence->data, first, count));
}
#endif


UINT2 XLALINT2SequenceSumSquares (
	const INT2Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALINT2SumSquares (sequence->data, first, count));
}





void XLALDestroyUINT2Sequence (
	UINT2Sequence *sequence
)
{
	if(sequence)
		LALFree(sequence->data);
	LALFree(sequence);
}


UINT2Sequence *XLALCreateUINT2Sequence (
	size_t length
)
{
	static const char *func = "XLALCreateUINT2Sequence";
	UINT2Sequence *new;
	UINT2 *data;

	new = LALMalloc(sizeof(*new));
	data = LALMalloc(length * sizeof(*data));
	if(!new || !data) {
		LALFree(new);
		LALFree(data);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	new->data = data;
	new->length = length;

	return(new);
}


UINT2Sequence *XLALCutUINT2Sequence (
	UINT2Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutUINT2Sequence";
	UINT2Sequence *new = NULL;

	if(sequence && sequence->data) {
		new = XLALCreateUINT2Sequence (length);
		if(!new)
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
		memcpy(new->data, sequence->data + first, length * sizeof(*new->data));
	}

	return(new);
}


UINT2Sequence *XLALCopyUINT2Sequence (
	UINT2Sequence *sequence
)
{
	return(XLALCutUINT2Sequence (sequence, 0, sequence->length));
}


void XLALShiftUINT2Sequence (
	UINT2Sequence *sequence,
	int count
)
{
	if(!sequence || !sequence->data || !count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
/* FIXME: this function does not conform to the XLAL error reporting
 * convention. */
size_t XLALResizeUINT2Sequence (
	UINT2Sequence *sequence,
	int first,
	size_t length
)
{
	if(!sequence || !sequence->data)
		return(0);

	if(length > sequence->length) {
		/* need to increase memory */
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(sequence->data) {
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftUINT2Sequence (sequence, -first);
		} else
			sequence->length = 0;
	} else {
		/* do not need to increase memory */
		XLALShiftUINT2Sequence (sequence, -first);
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(sequence->data)
			sequence->length = length;
		else
			sequence->length = 0;
	}

	return(sequence->length);
}


size_t XLALShrinkUINT2Sequence (
	UINT2Sequence *sequence,
	size_t first,
	size_t length
)
{
	return(XLALResizeUINT2Sequence (sequence, first, length));
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
UINT2 XLALUINT2Sum (
	const UINT2 *data,
	size_t first,
	size_t count
)
{
	UINT2 sum = 0;

	for(data += first; count-- > 0; data++) {
		sum += *data;
	}

	return(sum);
}
#endif


UINT2 XLALUINT2SumSquares (
	const UINT2 *data,
	size_t first,
	size_t count
)
{
	UINT2 sum = 0;

	for(data += first; count-- > 0; data++) {
		sum += *data * *data;
	}

	return(sum);
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
UINT2 XLALUINT2SequenceSum (
	const UINT2Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALUINT2Sum (sequence->data, first, count));
}
#endif


UINT2 XLALUINT2SequenceSumSquares (
	const UINT2Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALUINT2SumSquares (sequence->data, first, count));
}





void XLALDestroyINT4Sequence (
	INT4Sequence *sequence
)
{
	if(sequence)
		LALFree(sequence->data);
	LALFree(sequence);
}


INT4Sequence *XLALCreateINT4Sequence (
	size_t length
)
{
	static const char *func = "XLALCreateINT4Sequence";
	INT4Sequence *new;
	INT4 *data;

	new = LALMalloc(sizeof(*new));
	data = LALMalloc(length * sizeof(*data));
	if(!new || !data) {
		LALFree(new);
		LALFree(data);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	new->data = data;
	new->length = length;

	return(new);
}


INT4Sequence *XLALCutINT4Sequence (
	INT4Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutINT4Sequence";
	INT4Sequence *new = NULL;

	if(sequence && sequence->data) {
		new = XLALCreateINT4Sequence (length);
		if(!new)
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
		memcpy(new->data, sequence->data + first, length * sizeof(*new->data));
	}

	return(new);
}


INT4Sequence *XLALCopyINT4Sequence (
	INT4Sequence *sequence
)
{
	return(XLALCutINT4Sequence (sequence, 0, sequence->length));
}


void XLALShiftINT4Sequence (
	INT4Sequence *sequence,
	int count
)
{
	if(!sequence || !sequence->data || !count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
/* FIXME: this function does not conform to the XLAL error reporting
 * convention. */
size_t XLALResizeINT4Sequence (
	INT4Sequence *sequence,
	int first,
	size_t length
)
{
	if(!sequence || !sequence->data)
		return(0);

	if(length > sequence->length) {
		/* need to increase memory */
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(sequence->data) {
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftINT4Sequence (sequence, -first);
		} else
			sequence->length = 0;
	} else {
		/* do not need to increase memory */
		XLALShiftINT4Sequence (sequence, -first);
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(sequence->data)
			sequence->length = length;
		else
			sequence->length = 0;
	}

	return(sequence->length);
}


size_t XLALShrinkINT4Sequence (
	INT4Sequence *sequence,
	size_t first,
	size_t length
)
{
	return(XLALResizeINT4Sequence (sequence, first, length));
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
INT4 XLALINT4Sum (
	const INT4 *data,
	size_t first,
	size_t count
)
{
	INT4 sum = 0;

	for(data += first; count-- > 0; data++) {
		sum += *data;
	}

	return(sum);
}
#endif


UINT4 XLALINT4SumSquares (
	const INT4 *data,
	size_t first,
	size_t count
)
{
	UINT4 sum = 0;

	for(data += first; count-- > 0; data++) {
		sum += *data * *data;
	}

	return(sum);
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
INT4 XLALINT4SequenceSum (
	const INT4Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALINT4Sum (sequence->data, first, count));
}
#endif


UINT4 XLALINT4SequenceSumSquares (
	const INT4Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALINT4SumSquares (sequence->data, first, count));
}





void XLALDestroyUINT4Sequence (
	UINT4Sequence *sequence
)
{
	if(sequence)
		LALFree(sequence->data);
	LALFree(sequence);
}


UINT4Sequence *XLALCreateUINT4Sequence (
	size_t length
)
{
	static const char *func = "XLALCreateUINT4Sequence";
	UINT4Sequence *new;
	UINT4 *data;

	new = LALMalloc(sizeof(*new));
	data = LALMalloc(length * sizeof(*data));
	if(!new || !data) {
		LALFree(new);
		LALFree(data);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	new->data = data;
	new->length = length;

	return(new);
}


UINT4Sequence *XLALCutUINT4Sequence (
	UINT4Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutUINT4Sequence";
	UINT4Sequence *new = NULL;

	if(sequence && sequence->data) {
		new = XLALCreateUINT4Sequence (length);
		if(!new)
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
		memcpy(new->data, sequence->data + first, length * sizeof(*new->data));
	}

	return(new);
}


UINT4Sequence *XLALCopyUINT4Sequence (
	UINT4Sequence *sequence
)
{
	return(XLALCutUINT4Sequence (sequence, 0, sequence->length));
}


void XLALShiftUINT4Sequence (
	UINT4Sequence *sequence,
	int count
)
{
	if(!sequence || !sequence->data || !count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
/* FIXME: this function does not conform to the XLAL error reporting
 * convention. */
size_t XLALResizeUINT4Sequence (
	UINT4Sequence *sequence,
	int first,
	size_t length
)
{
	if(!sequence || !sequence->data)
		return(0);

	if(length > sequence->length) {
		/* need to increase memory */
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(sequence->data) {
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftUINT4Sequence (sequence, -first);
		} else
			sequence->length = 0;
	} else {
		/* do not need to increase memory */
		XLALShiftUINT4Sequence (sequence, -first);
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(sequence->data)
			sequence->length = length;
		else
			sequence->length = 0;
	}

	return(sequence->length);
}


size_t XLALShrinkUINT4Sequence (
	UINT4Sequence *sequence,
	size_t first,
	size_t length
)
{
	return(XLALResizeUINT4Sequence (sequence, first, length));
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
UINT4 XLALUINT4Sum (
	const UINT4 *data,
	size_t first,
	size_t count
)
{
	UINT4 sum = 0;

	for(data += first; count-- > 0; data++) {
		sum += *data;
	}

	return(sum);
}
#endif


UINT4 XLALUINT4SumSquares (
	const UINT4 *data,
	size_t first,
	size_t count
)
{
	UINT4 sum = 0;

	for(data += first; count-- > 0; data++) {
		sum += *data * *data;
	}

	return(sum);
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
UINT4 XLALUINT4SequenceSum (
	const UINT4Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALUINT4Sum (sequence->data, first, count));
}
#endif


UINT4 XLALUINT4SequenceSumSquares (
	const UINT4Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALUINT4SumSquares (sequence->data, first, count));
}





void XLALDestroyINT8Sequence (
	INT8Sequence *sequence
)
{
	if(sequence)
		LALFree(sequence->data);
	LALFree(sequence);
}


INT8Sequence *XLALCreateINT8Sequence (
	size_t length
)
{
	static const char *func = "XLALCreateINT8Sequence";
	INT8Sequence *new;
	INT8 *data;

	new = LALMalloc(sizeof(*new));
	data = LALMalloc(length * sizeof(*data));
	if(!new || !data) {
		LALFree(new);
		LALFree(data);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	new->data = data;
	new->length = length;

	return(new);
}


INT8Sequence *XLALCutINT8Sequence (
	INT8Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutINT8Sequence";
	INT8Sequence *new = NULL;

	if(sequence && sequence->data) {
		new = XLALCreateINT8Sequence (length);
		if(!new)
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
		memcpy(new->data, sequence->data + first, length * sizeof(*new->data));
	}

	return(new);
}


INT8Sequence *XLALCopyINT8Sequence (
	INT8Sequence *sequence
)
{
	return(XLALCutINT8Sequence (sequence, 0, sequence->length));
}


void XLALShiftINT8Sequence (
	INT8Sequence *sequence,
	int count
)
{
	if(!sequence || !sequence->data || !count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
/* FIXME: this function does not conform to the XLAL error reporting
 * convention. */
size_t XLALResizeINT8Sequence (
	INT8Sequence *sequence,
	int first,
	size_t length
)
{
	if(!sequence || !sequence->data)
		return(0);

	if(length > sequence->length) {
		/* need to increase memory */
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(sequence->data) {
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftINT8Sequence (sequence, -first);
		} else
			sequence->length = 0;
	} else {
		/* do not need to increase memory */
		XLALShiftINT8Sequence (sequence, -first);
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(sequence->data)
			sequence->length = length;
		else
			sequence->length = 0;
	}

	return(sequence->length);
}


size_t XLALShrinkINT8Sequence (
	INT8Sequence *sequence,
	size_t first,
	size_t length
)
{
	return(XLALResizeINT8Sequence (sequence, first, length));
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
INT8 XLALINT8Sum (
	const INT8 *data,
	size_t first,
	size_t count
)
{
	INT8 sum = 0;

	for(data += first; count-- > 0; data++) {
		sum += *data;
	}

	return(sum);
}
#endif


UINT8 XLALINT8SumSquares (
	const INT8 *data,
	size_t first,
	size_t count
)
{
	UINT8 sum = 0;

	for(data += first; count-- > 0; data++) {
		sum += *data * *data;
	}

	return(sum);
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
INT8 XLALINT8SequenceSum (
	const INT8Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALINT8Sum (sequence->data, first, count));
}
#endif


UINT8 XLALINT8SequenceSumSquares (
	const INT8Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALINT8SumSquares (sequence->data, first, count));
}





void XLALDestroyUINT8Sequence (
	UINT8Sequence *sequence
)
{
	if(sequence)
		LALFree(sequence->data);
	LALFree(sequence);
}


UINT8Sequence *XLALCreateUINT8Sequence (
	size_t length
)
{
	static const char *func = "XLALCreateUINT8Sequence";
	UINT8Sequence *new;
	UINT8 *data;

	new = LALMalloc(sizeof(*new));
	data = LALMalloc(length * sizeof(*data));
	if(!new || !data) {
		LALFree(new);
		LALFree(data);
		XLAL_ERROR_NULL(func, XLAL_ENOMEM);
	}

	new->data = data;
	new->length = length;

	return(new);
}


UINT8Sequence *XLALCutUINT8Sequence (
	UINT8Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char *func = "XLALCutUINT8Sequence";
	UINT8Sequence *new = NULL;

	if(sequence && sequence->data) {
		new = XLALCreateUINT8Sequence (length);
		if(!new)
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
		memcpy(new->data, sequence->data + first, length * sizeof(*new->data));
	}

	return(new);
}


UINT8Sequence *XLALCopyUINT8Sequence (
	UINT8Sequence *sequence
)
{
	return(XLALCutUINT8Sequence (sequence, 0, sequence->length));
}


void XLALShiftUINT8Sequence (
	UINT8Sequence *sequence,
	int count
)
{
	if(!sequence || !sequence->data || !count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
/* FIXME: this function does not conform to the XLAL error reporting
 * convention. */
size_t XLALResizeUINT8Sequence (
	UINT8Sequence *sequence,
	int first,
	size_t length
)
{
	if(!sequence || !sequence->data)
		return(0);

	if(length > sequence->length) {
		/* need to increase memory */
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(sequence->data) {
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftUINT8Sequence (sequence, -first);
		} else
			sequence->length = 0;
	} else {
		/* do not need to increase memory */
		XLALShiftUINT8Sequence (sequence, -first);
		sequence->data = LALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(sequence->data)
			sequence->length = length;
		else
			sequence->length = 0;
	}

	return(sequence->length);
}


size_t XLALShrinkUINT8Sequence (
	UINT8Sequence *sequence,
	size_t first,
	size_t length
)
{
	return(XLALResizeUINT8Sequence (sequence, first, length));
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
UINT8 XLALUINT8Sum (
	const UINT8 *data,
	size_t first,
	size_t count
)
{
	UINT8 sum = 0;

	for(data += first; count-- > 0; data++) {
		sum += *data;
	}

	return(sum);
}
#endif


UINT8 XLALUINT8SumSquares (
	const UINT8 *data,
	size_t first,
	size_t count
)
{
	UINT8 sum = 0;

	for(data += first; count-- > 0; data++) {
		sum += *data * *data;
	}

	return(sum);
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
UINT8 XLALUINT8SequenceSum (
	const UINT8Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALUINT8Sum (sequence->data, first, count));
}
#endif


UINT8 XLALUINT8SequenceSumSquares (
	const UINT8Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return(0);
	if(first + count > sequence->length)
		count = sequence->length - first;
	return(XLALUINT8SumSquares (sequence->data, first, count));
}

